/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm.model;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 *
 * @author Timon Veenstra & Gerben Feenstra
 */
@Messages({
    "FarmDetailsPanel.icon.toolTipText=Farm Details",
    "FarmDetailsPanel.icon.text=",
    "FarmDetailsPanel.nameLabel.toolTipText=Company name",
    "FarmDetailsPanel.nameLabel.text=Company name:",
    "FarmDetailsPanel.registrationNumberLabel.toolTipText=Registration number",
    "FarmDetailsPanel.registrationNumberLabel.text=Registration number:"
})
public class FarmDetailsPanel extends ASBodyPanel implements PropertyChangeListener {

    private ImageIcon titleIcon = new ImageIcon(ImageUtilities.loadImage("nl/cloudfarming/client/farm/model/house32.png"));
    private static final int FIELD_WIDTH = 250;
    private static final String ICON_LABEL_TEXT = NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.icon.text");
    private static final String ICON_LABEL_TOOLTIP_TEXT = NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.icon.toolTipText");
    private static final String NAME_LABEL_TEXT = NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.nameLabel.text");
    private static final String NAME_LABEL_TOOLTIP_TEXT = NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.nameLabel.toolTipText");
    private static final String REGISTRATION_LABEL_TEXT = NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.registrationNumberLabel.text");
    private static final String REGISTRATION_LABEL_TOOLTIP_TEXT = NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.registrationNumberLabel.toolTipText");
    private Farm farm = new Farm();
    // indicating change from the panel to the farm
    private boolean outboundChange = false;
    // indicating change from the farm to the panel
    private boolean inboundChange = false;
    private JTextField farmNameTextField = new JTextField();
    private JTextField farmRegistrationNumberTextField = new JTextField();
    private JLabel farmIconLabel = new JLabel();
    private JLabel farmNameLabel = new JLabel();
    private JLabel farmRegistrationNumberLabel = new JLabel();

    /**
     * Creates new form FarmWizardDetailsVisual
     */
    public FarmDetailsPanel(Farm farm) {
        super(new MigLayout("wrap 2"));
        this.farm = farm;
        initComponents();

        // Add listeners
        farmNameTextField.getDocument().addDocumentListener(new NameChangeListener());
        farmRegistrationNumberTextField.getDocument().addDocumentListener(new RegistrationChangeListener());

        farm.addPropertyChangeListener(this);
    }

    public Farm getFarm() {
        return farm;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     */
    private void initComponents() {
        
        setIcon(titleIcon);
        farmIconLabel.setText(ICON_LABEL_TEXT);
        farmIconLabel.setToolTipText(ICON_LABEL_TOOLTIP_TEXT);

        farmNameLabel.setText(NAME_LABEL_TEXT);
        farmNameLabel.setToolTipText(NAME_LABEL_TOOLTIP_TEXT);

        farmNameTextField.setText(farm.getName());
        farmNameTextField.setPreferredSize(new Dimension(FIELD_WIDTH, 0));

        farmRegistrationNumberLabel.setText(REGISTRATION_LABEL_TEXT);
        farmRegistrationNumberLabel.setToolTipText(REGISTRATION_LABEL_TOOLTIP_TEXT);

        farmRegistrationNumberTextField.setText(farm.getRegistrationNumber());
        farmRegistrationNumberTextField.setPreferredSize(new Dimension(FIELD_WIDTH, 0));

        this.add(farmNameLabel);
        this.add(farmNameTextField);
        this.add(farmRegistrationNumberLabel);
        this.add(farmRegistrationNumberTextField);
        
        setTitle(farm.getName());
        
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (!outboundChange) {
            inboundChange = true;
            if (Farm.PROP_NAME.equals(evt.getPropertyName())) {
                farmNameTextField.setText((String) evt.getNewValue());
                setTitle((String) evt.getNewValue());
            }
            if (Farm.PROP_REGISTRATIONNUMBER.equals(evt.getPropertyName())) {
                farmRegistrationNumberTextField.setText((String) evt.getNewValue());
            }
            inboundChange = false;
        }
    }

    private class NameChangeListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            updateName(e.getDocument());
            updateTitle(e.getDocument());
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            updateName(e.getDocument());
            updateTitle(e.getDocument());
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            updateName(e.getDocument());
            updateTitle(e.getDocument());
        }

        void updateName(Document d) {
            if (!inboundChange) {
                outboundChange = true;
                try {
                    farm.setName(d.getText(0, d.getLength()));
                } catch (BadLocationException ex) {
                    Exceptions.printStackTrace(ex);
                }
                outboundChange = false;
            }
        }

        void updateTitle(Document d) {
            if (!inboundChange) {
                outboundChange = true;
                try {
                    FarmDetailsPanel.this.setTitle(d.getText(0, d.getLength()));
                } catch (BadLocationException ex) {
                    Exceptions.printStackTrace(ex);
                }
                outboundChange = false;
            }
        }
    }

    private class RegistrationChangeListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        void updateName(Document d) {
            if (!inboundChange) {
                outboundChange = true;
                try {
                    farm.setRegistrationNumber(d.getText(0, d.getLength()));
                } catch (BadLocationException ex) {
                    Exceptions.printStackTrace(ex);
                }
                outboundChange = false;
            }
        }
    }
}
